@@ -48,7 +48,7 @@ def _filter_content( rawString ):
             filename, internal, mimetype = None, None, None
         # Read next line
         (line, nextpos) = _readline( rawString, nextpos )
-        if line.startswith("END_OF_HEADER"):
+        if line.startswith("</pseudoformatter_header>"):
             break
         if line.startswith("ATTACHMENT:"):
             filename = line[ len("ATTACHMENT:") : ]
@@ -103,7 +103,7 @@ def execute(pagename, request):
     page = Page(request, pagename, formatter = formatter)
     
     (attachments, contentString) = \
-        _filter_content( request.redirectedOutput(page.send_page, request, emit_headers=0, count_hit=0).encode( 'utf-8' ) )
+        _filter_content( request.redirectedOutput(page.send_page, emit_headers=0, count_hit=0).encode( 'utf-8' ) )
     
     info = zipfile.ZipInfo("content.xml", now)
     info.compress_type = zipfile.ZIP_DEFLATED     # ZIP_DEFLATED = compressed
