@@ -37,11 +37,14 @@ E_FORMATTING = '__FORMAT'
 # Element for text
 E_TEXT = '__TEXT'
 
+# Element for raw XML, e.g. content of included pages
+E_RAW = '__RAW'
+
 # Lists of direct subelements
 validChildren = {'office:document-content': ['office:body', 'office:automatic-styles'], \
-                            'office:automatic-styles': ['style:style'], \
+                            'office:automatic-styles': ['style:style', E_RAW], \
                             'office:body': ['office:text'], \
-                            'office:text': ['text:h', 'text:p', 'table:table', 'text:list' ], \
+                            'office:text': ['text:h', 'text:p', 'table:table', 'text:list', E_RAW], \
                             'style:style': ['style:paragraph-properties', 'style:text-properties'], \
                             'text:h': [ E_TEXT ], \
                             'text:list' : [ 'text:list-item' ], \
@@ -63,6 +66,14 @@ validGrandChildren = { 'table:table-cell': { E_TEXT: 'text:p' }, \
 # paragraphs where a first single space character is always ignored)
 whitespaceSearch = re.compile('(  +|^ )')  
 
+# escape HTML / XML text
+def _escape(text):
+        text = text.replace('&','&amp;')
+        text = text.replace("'", '&apos;')
+        text = text.replace('"', '&quot;')
+        text = text.replace('<','&lt;')
+        text = text.replace('>','&gt;')
+        return text
 
 
 # ---------------------------------------------- Node Class  ---------------------------------------------------
@@ -118,11 +129,7 @@ class Node:
         # Don't add just nothing
         if len(text) == 0: return
         # Excape XML
-        text = text.replace('&','&amp;')
-        text = text.replace("'", '&apos;')
-        text = text.replace('"', '&quot;')
-        text = text.replace('<','&lt;')
-        text = text.replace('>','&gt;')
+        text = _escape(text)
         # Add Text Node
         newNode = Node(E_TEXT)
         newNode.content = text
@@ -186,7 +193,7 @@ class Node:
 
     def to_XML_list(self, liste ):
 
-        if self.element == E_TEXT:
+        if self.element == E_TEXT or self.element == E_RAW:
             liste.append( self.content )
             return
             
@@ -223,6 +230,9 @@ class Formatter(FormatterBase):
         self.styleNames = []                # Names of children of self.stylesNode
         self.attachmentCounter = 0   # Replace Attachment File names with Counter + File Extension
         self.attachments = {}             # Real file name, internal file name (s.a.)
+        self.include_headers = ""        # Attachments of included pages
+        self.include_counter = 0         # Count Included Pages
+        self.isIncluded = kw.get('is_included', False)
 
 
     # Helper Method: get next attachment counter
@@ -283,6 +293,8 @@ class Formatter(FormatterBase):
             attribList = [ attrib for attrib in currentNode.attributes ]
             attribList.sort()
             styleName = 'AS_' + '_'.join( attribList )
+            if self.isIncluded:
+                styleName = '<<INCLUDE_ID>>' + styleName  # Prefix will be replaced by calling formatter
             # Convert node into text:span
             currentNode.element = 'text:span'
             currentNode.attributes = {}
@@ -359,13 +371,17 @@ class Formatter(FormatterBase):
 
     # Helper method: render internal header (only for action class RenderAsOpenDocument)
     def getInternalHeader(self):
-        result = "BEGIN_OF_HEADER\n"
+        result = "\n<pseudoformatter_header>\n"
         
         for at in self.attachments:
             result += "ATTACHMENT:" + at + "\n" 
             result += "INTERNAL:" + self.attachments[ at ] + "\n"
             result += "MIMETYPE:" + self.getMimetype( at )[0] + "\n"
-        result += "END_OF_HEADER\n"
+
+        # Add Headers of included pages
+        result += '\n' + self.include_headers
+
+        result += "</pseudoformatter_header>\n"
         return result
 
 
@@ -396,7 +412,7 @@ class Formatter(FormatterBase):
         # From now on we can use the automated way of node insertion
         self.open( "office:body" )
         self.open( "office:text" )
-
+        
         return ''
 
 
@@ -408,12 +424,27 @@ class Formatter(FormatterBase):
         return self.getInternalHeader() + ''.join( liste )
 
 
+    # Embedded Mode (e.g. Include Macro)
+    def startContent(self, content_id="content", **kw):
+        if self.isIncluded:
+            return self.startDocument(self.page.page_name)
+        # Without this line, TableOfContents-Macro fails
+        return FormatterBase.startContent(self, content_id, **kw)
+
+
+    # Embedded Mode (e.g. Include Macro)
+    def endContent(self):
+        if self.isIncluded:
+            return self.endDocument()
+        return FormatterBase.endContent(self)
+
+
     # The MoinMoin drawing extension creates a PNG file besides the vector file
     def attachment_drawing(self, url, *args, **kargs): 
         self.attachment_image( url + '.png', *args, **kargs)
         return ""
 
-    
+
     def attachment_image(self, url, *args, **kargs): 
         # prevent hacking (filenames like "../../../../etc/passwd")
         attachbasename = wikiutil.taintfilename( url )
@@ -427,16 +458,22 @@ class Formatter(FormatterBase):
 
         # create internal name and store filename->internal name mapping, but only once
         if attachfilename in self.attachments:
-            internalName = self.attachmants[ attachfilename ]
+            internalName = self.attachments[ attachfilename ]
         else:
+            if self.isIncluded:
+                # Will be replaced by calling formatter
+                includeId = '<<INCLUDE_ID>>'
+            else:
+                includeId = ''
+                
             internalName = str( self.getNextAttachmentCounter() )
-            internalName = "Pictures/" + (12 - len(internalName) ) * "0" + internalName + "." + extension
+            internalName = "Pictures/" + includeId + (12 - len(internalName) ) * "0" + internalName + "." + extension
             self.attachments[ attachfilename ] = internalName
 
         # embed image
         self.open( 'draw:frame' )
         imageNode = self.open( 'draw:image' )
-        imageNode.setAttribute( 'xlink:href', internalName )
+        imageNode.setAttribute( 'xlink:href', self.attachments[attachfilename] )
         imageNode.setAttribute( 'xlink:type', 'simple' )
         imageNode.setAttribute( 'xlink:show', 'embed' )
         imageNode.setAttribute( 'xlink:actuate', 'onLoad' )
@@ -542,7 +579,66 @@ class Formatter(FormatterBase):
         else:
             self.close( 'text:list-item' )
         return ""
-    
+
+
+    def macro(self, macro_obj, name, args, markup=None):
+        if name == "Include":
+            # Include-Macro
+            
+            rawXML = FormatterBase.macro(self, macro_obj, name, args)
+            
+            # Include-ID for embedded page
+            self.include_counter += 1
+            
+            if self.isIncluded:
+                # If we are included as well: Don't remove placeholder <<INCLUDE_ID>>,
+                # but extend it with IncludeId of embedded page
+                replacement = '<<INCLUDE_ID>>INC' + str(self.include_counter) + '_'
+            else:
+                # If we are not included: Replace <<INCLUDE_ID>> with IncludeId of embedded page
+                replacement = 'INC' + str(self.include_counter) + '_'
+            
+            rawXML = rawXML.replace('<<INCLUDE_ID>>', replacement)
+            
+            # Get Header. Since this is not really build from XML tree, it's never an empty tag
+            searchL = '<pseudoformatter_header>\n'
+            searchR = '</pseudoformatter_header>'
+            self.include_headers += rawXML [ rawXML.find( searchL ) + len(searchL) :  rawXML.find( searchR ) ]
+
+            # Search for Automatic Styles
+            searchL = '<office:automatic-styles>'
+            posL = rawXML.find( searchL )
+            # Avoid empty <office:automatic-styles/>
+            if posL != -1:
+                searchR = '</office:automatic-styles>'
+                rawAutomaticStyles = rawXML [ posL + len(searchL) : rawXML.find( searchR ) ].strip()
+                if len(rawAutomaticStyles):
+                    rawNode = Node( E_RAW )
+                    rawNode.content = rawAutomaticStyles
+                    self.stylesNode.addChild( rawNode )
+                    
+            # Get office:text content
+            searchL = '<office:text>'
+            posL = rawXML.find( searchL )
+            # Avoid empty <office:text/>
+            if posL == -1:   
+                return ''
+            searchR = '</office:text>'
+            rawOfficeText = rawXML [ posL + len(searchL) : rawXML.find( searchR ) ]
+            # Return from Lists, paragraphs, etc.
+            while self.Node.element != 'office:text' and self.Node.parent:
+                self.Node = self.Node.parent
+            # Add Raw Node
+            rawNode = Node( E_RAW )
+            rawNode.content = rawOfficeText
+            self.Node.addChild( rawNode )
+            return ""
+            
+        else:
+            
+            # Other Macro
+            return FormatterBase.macro(self, macro_obj, name, args)
+
     
     def number_list(self, on, *args, **kargs): 
         if on:
@@ -735,7 +831,7 @@ class Formatter(FormatterBase):
         if on:
             newNode = self.open("text:a")
             newNode.setAttribute("xlink:type", "simple")
-            newNode.setAttribute("xlink:href", url)
+            newNode.setAttribute("xlink:href", _escape(url) )
         else:
             self.close("text:a")
         return ""
